/**
  ******************************************************************************
  * @file    Project/main.c
  * @author  "Vladimir N. Shilov" <shilow@ukr.net>
  * @version v.0.1
  * @date    17-April-2020
  * @brief   Main program body
   ******************************************************************************
  * @attention
  *
  * "THE BEER-WARE LICENSE" (Revision 42):
  * <shilow@ukr.net> wrote this file.  As long as you retain this notice you
  * can do whatever you want with this stuff. If we meet some day, and you think
  * this stuff is worth it, you can buy me a beer in return.   Shilov V.N.
  *
  ******************************************************************************
  */


/* Includes ------------------------------------------------------------------*/
#include <stm8s.h>
#include "board.h"
#include "rtos.h"
#include "i2c.h"

/* Private define ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private constants ---------------------------------------------------------*/
/* Public variables ----------------------------------------------------------*/
uint8_t irBuf[12];
uint8_t irFlag = 0;

/* Private variables ---------------------------------------------------------*/
static uint8_t Volume;
static uint8_t muteVolume;
static ir_btn_t irBtn = {0, 0, 0};

/* EEPROM variables ----------------------------------------------------------*/
#define EEP1_ADDR FLASH_DATA_START_PHYSICAL_ADDRESS
#pragma location=EEP1_ADDR
__no_init uint8_t EEP_VOLUME;
//#pragma required=EEP_VOLUME

/* Private function prototypes -----------------------------------------------*/
static void boardInit(void);
static void eepromStore(void);
static void volumeSet(uint8_t newvol, uint8_t save);
static void volumeMute(void);
static void checkMute(void);

/* RTOS function prototypes -----------------------------------------------*/
static void encoderCheck(void);
static void btnCheck(void);
static void irBtnCheck(void);
static void irSeqReset(void);

void main(void)
{
  /* Board Configuration */
  boardInit();

  /* RTOS Configuration */
  RTOS_Init();

  /* I2C Configuration */
  i2c_master_init();

  /* ROTS tasks */
  RTOS_SetTask(encoderCheck, 0, 50);
  RTOS_SetTask(btnCheck, 1, BTN_SCAN_PERIOD);
  //RTOS_SetTask(irBtnCheck, 3, BTN_SCAN_PERIOD);

  Volume = EEP_VOLUME;
  volumeSet(Volume, 0);

  uint8_t ir_btn;
  /* Infinite loop */
  while (1) {

    RTOS_DispatchTask();

    wfi();

    if (irFlag != 0) {
      if (irBuf[0] >= IR_THRESHOLD) {
        irBtn.code = 0x04;
      }
      if (irBuf[1] >= IR_THRESHOLD) {
        irBtn.code |= 0x02;
      }
      if (irBuf[2] >= IR_THRESHOLD) {
        irBtn.code |= 0x01;
      }

      if (irBuf[3] >= IR_THRESHOLD) {
        ir_btn = 0;
      }
      if (irBuf[4] >= IR_THRESHOLD) {
        ir_btn = 6;
      }
      if (irBuf[5] >= IR_THRESHOLD) {
        ir_btn = 12;
      }

      uint8_t i;
      for (i=6;i<12;i++) {
        if (irBuf[i] >= IR_THRESHOLD) {
          ir_btn += (i - 5);
        }
      }

      irFlag = 0;

      if (irBtn.seq != 0) {
        irBtn.seq = 0;
        if (irBtn.key == ir_btn) {
          irBtnCheck();
        }
      } else {
        irBtn.key = ir_btn;
        irBtn.seq = 1;
        RTOS_SetTask(irSeqReset, 60, 0);
      }
    } /* End of irFlag != 0 */

  } /* End infinity loop */

}

/* Private functions ---------------------------------------------------------*/
static void boardInit(void) {
  /* Насколько я понял, для частот выше 16МГц нужно вводить задержку при работе с FLASH:
    - Before using the HSE clock make sure that the "Flash_Wait_States" is set to 1.
    - To do so :
     - with STVD (menu: Debug Instrument -> MCU configuration -> Options)
     - with EWSTM8 (menu: ST-LINK -> Option bytes ->Flash_Wait_States: 1)
  */

  /* Initialization of the clock to 16MHz */
  CLK->CKDIVR = 0x00;

  /* Disable clock of unused peripherial */
  CLK->PCKENR1 = (uint8_t)(~(CLK_PCKENR1_UART1 | CLK_PCKENR1_SPI));
  CLK->PCKENR2 = (uint8_t)(~(CLK_PCKENR2_CAN | CLK_PCKENR2_ADC | CLK_PCKENR2_AWU));

  /* Configure GPIO for Encoder */
  ENC_PORT->DDR &= (uint8_t)(~ENC_PIN_ALL); // Input Mode
  ENC_PORT->CR1 |= (uint8_t)ENC_PIN_ALL; // Pull-Up
  ENC_PORT->CR2 &= (uint8_t)(~ENC_PIN_ALL); // No Interrupt

  // test  pin
  GPIOC->DDR |= GPIO_PIN_3; // output
  GPIOC->CR1 |= GPIO_PIN_3; // push-pull
  GPIOC->CR2 |= GPIO_PIN_3; // fast mode
  GPIOC->ODR |= GPIO_PIN_3;

  /** Init Timer1 */
  /* Set the Autoreload value */
  TIM1->ARRH = (uint8_t)(TIM1_PERIOD >> 8);
  TIM1->ARRL = (uint8_t)(TIM1_PERIOD);
  /* Set the Prescaler value */
//  TIM1->PSCRH = 0;
//  TIM1->PSCRL = 0;

  /* Set the TI1 and the TI2 Polarities - TIM1_ICPOLARITY_RSISING */
 // TIM1->CCER1 = (TIM1_CCER1_CC2P | TIM1_CCER1_CC1P); // Change polarity
  /* Set the encoder Mode - TIM1_ENCODERMODE_TI12 */
  TIM1->SMCR = (uint8_t)TIM1_ENC_MODE;
  /* Select the Capture Compare 1 and the Capture Compare 2 as input */
  TIM1->CCMR1 = 0x01; // no filter, no prescaler, IC1 is mapped on TI1FP1
  TIM1->CCMR2 = 0x01; // no filter, no prescaler, IC2 is mapped on TI2FP2
  /* TIM1 Enable */
  TIM1->CR1 |= TIM1_CR1_CEN;

  /** Init Timer2 */
  // no filter, no prescaler, IC1 is mapped on TI1FP1
  TIM2->CCMR1 = 0x01;
  // no filter, no prescaler, IC2 is mapped on TI1FP2
  TIM2->CCMR2 = 0x02;
  // CC1 - falling edge, CC2 - rising edge, Capture enabled
  TIM2->CCER1 = (TIM2_CCER1_CC2E | TIM2_CCER1_CC1E | TIM2_CCER1_CC1P);
  // prescaler = 4096
  TIM2->PSCR = 12;
  // Set the Autoreload value
  TIM2->ARRH = 0x00;
  TIM2->ARRL = 0xFF;
  // Interrupts enable
  TIM2->IER = (TIM2_IER_CC2IE | TIM2_IER_CC1IE);
  // TIM2 Enable
  TIM2->CR1 |= TIM2_CR1_CEN;
}

static void encoderCheck(void) {
  uint8_t vol;
  uint16_t present_value;
  static uint16_t previous_value = 0;

  present_value = ((uint16_t)TIM1->CNTRH << 8);
  present_value |= (uint16_t)(TIM1->CNTRL);
  present_value >>= 2;

  if (present_value != previous_value) {

    checkMute();

    if (previous_value < present_value) {
      // clockwise rotate -- increase volume (0 = max)
      vol = present_value - previous_value;
      if (Volume >= vol) {
        Volume -= vol;
      } else {
        Volume = 0;
      }
    } else {
      // counterclockwise rotate -- decrease volume (TDA7448_MIN_VOL = min)
      vol = previous_value - present_value;
      Volume += vol;
      if (Volume > TDA7448_MIN_VOL) {
        Volume = TDA7448_MIN_VOL;
      }
    }

    volumeSet(Volume, 1);

    previous_value = present_value;
  }
}

static void btnCheck(void) {
  // if BTN pressed
  if ((ENC_PORT->IDR & ENC_PIN_BTN) == 0) {
    // no BTN scan for 200 msek
    RTOS_SetTask(btnCheck, BTN_NOSCAN_MS, BTN_SCAN_PERIOD);
    volumeMute();
  }
}

static void irBtnCheck(void) {
  // if new BTN code received
  if ((irBtn.key) != 0) {
    switch (irBtn.key) {

    case IR_BTN_POWER:
      break;

    case IR_BTN_MUTE:
      volumeMute();
      break;

    case IR_BTN_VOLP:
      checkMute();
      if (Volume > 0) {
        Volume --;
        volumeSet(Volume, 1);
      }
      break;

    case IR_BTN_VOLM:
      checkMute();
      if (Volume < TDA7448_MIN_VOL) {
        Volume ++;
        volumeSet(Volume, 1);
      }
      break;

    default:
      break;
    }

    irBtn.key = 0;
    irBtn.code = 0;
  }
}

static void irSeqReset(void) {
  irBtn.seq = 0;
  irBtn.key = 0;
}

static void volumeMute(void) {
  if (Volume == TDA7448_MUTE) {
    Volume = muteVolume;
    muteVolume = TDA7448_MUTE;
  } else {
    muteVolume = Volume;
    Volume = TDA7448_MUTE;
  }
  volumeSet(Volume, 1);
}

static void checkMute(void) {
  if (Volume == TDA7448_MUTE) {
    Volume = muteVolume;
    muteVolume = TDA7448_MUTE;
  }
}

/** Save Volume value to EEPROM */
static void eepromStore(void) {
  disableInterrupts();

  /* Unlock Data Flash */
  FLASH->DUKR = (uint8_t)0xAE;
  FLASH->DUKR = (uint8_t)0x56;

  /* wait for eeprom ready */
  while (!(FLASH->IAPSR & FLASH_IAPSR_DUL));

  EEP_VOLUME = Volume;

  /* Lock Flash */
  FLASH->IAPSR &= (uint8_t)0xF7;

  enableInterrupts();
}

static void volumeSet(uint8_t newvol, uint8_t save) {
  t_i2c_status i2c_status;
  uint8_t buf[6];

  if ((newvol > TDA7448_MIN_VOL) && (newvol != TDA7448_MUTE)) {
    newvol = TDA7448_MIN_VOL;
  }

  buf[0] = newvol;
  buf[1] = newvol;
  buf[2] = newvol;
  buf[3] = newvol;
  buf[4] = newvol;
  buf[5] = newvol;

  i2c_status = i2c_wr_reg(TDA7448_I2C_WRADDR, TDA7448_SUB_ADDR, buf, 6);
  if (i2c_status != I2C_SUCCESS) {
    if (i2c_status == I2C_TIMEOUT) {
      // indicate i2c timeout
      uint8_t c = 3;
      while (c) {
        GPIOC->ODR &= ~GPIO_PIN_3;
        Delay(1);
        GPIOC->ODR |= GPIO_PIN_3;
        Delay(1);
        c --;
      }
    } else {
      // indicate i2c error
      uint8_t c = 3;
      while (c) {
        GPIOC->ODR &= ~GPIO_PIN_3;
        Delay(3);
        GPIOC->ODR |= GPIO_PIN_3;
        Delay(1);
        c --;
      }
    }
  }

  if (save != 0) {
    RTOS_SetTask(eepromStore, 2000, 0); // After 2 sec store Volume to EEPROM
  }
}

#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
  */
void assert_failed(u8* file, u32 line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif


/************************ (C) COPYRIGHT Shilov V.N. *****END OF FILE****/
